

import oracle.toplink.essentials.threetier.ServerSession;

import javax.persistence.EntityManagerFactory;
import javax.persistence.EntityManager;
import javax.persistence.Persistence;
import javax.persistence.EntityTransaction;

import java.sql.Connection;
import java.sql.DriverManager;
import java.sql.SQLException;
import java.util.Map;
import java.util.Properties;
import java.io.BufferedInputStream;
import java.io.IOException;
import java.io.InputStream;
import java.io.PrintStream;
import java.net.URL;

/**
 * @author Wonseok Kim
 */
public class JpaTestCase {
    public static String DEFAULT_PROPERTIES_FILE = "META-INF/persistence.properties";
//    public static String _PROPERTIES_FILE = "persistence.properties";

    protected String puName;
    protected String propertiesFileName = DEFAULT_PROPERTIES_FILE;
    protected EntityManagerFactory emf;
    protected EntityManager em;

    protected static PrintStream out = System.out;
    protected Connection connection;

    protected Connection getConnection() throws SQLException {
        //get raw database connection
        if(connection == null) {
            Map map = getPersistenceProperties();
            String jdbcUrl = (String) map.get("toplink.jdbc.url");
            String jdbcUserid = (String) map.get("toplink.jdbc.user");
            String jdbcPassword = (String) map.get("toplink.jdbc.password");
            connection = DriverManager.getConnection(jdbcUrl, jdbcUserid, jdbcPassword);
            println("Successfully got database connection");
        }
        return connection;
    }

    protected void closeConnection() throws SQLException {
        if(connection != null)
            connection.close();
    }

    public JpaTestCase() {
        this.puName = "pu";
    }

    public JpaTestCase(String puName) {
        this.puName = puName;
    }

    protected void setPropertiesFileName(String name) {
        propertiesFileName = name;
    }
    
    private Map getPersistenceProperties() {
        return JpaTestHelper.getProperties(propertiesFileName);
    }
    
    public EntityManagerFactory getEntityManagerFactory() {
        if(emf == null)
            emf = Persistence.createEntityManagerFactory(puName, getPersistenceProperties());
        return emf;
    }

    public EntityManagerFactory createEntityManagerFactory(String unitName) {
        return Persistence.createEntityManagerFactory(unitName, getPersistenceProperties());
    }
    
    public EntityManager getEntityManager(){
        if(em == null){
            em = getEntityManagerFactory().createEntityManager();
        }
        return em;
    }

    public EntityManager createEntityManager(){
        return getEntityManagerFactory().createEntityManager();
    }

    public ServerSession getServerSession() {
        return ((oracle.toplink.essentials.internal.ejb.cmp3.EntityManagerImpl)getEntityManager()).getServerSession();               
    }

    public void clearCache() {
         try {
            getServerSession().getIdentityMapAccessor().initializeAllIdentityMaps();
         } catch (Exception ex) {
            throw new RuntimeException("An exception occurred trying clear the cache.", ex);
        }   
    }

    public void clearCache(EntityManager em) {
         try {
            ((oracle.toplink.essentials.internal.ejb.cmp3.EntityManagerImpl)em).getServerSession().getIdentityMapAccessor().initializeIdentityMaps();
         } catch (Exception ex) {
            throw new RuntimeException("An exception occurred trying clear the cache.", ex);
        }   
    }
    
    public void begin(){
        getEntityManager();
        em.getTransaction().begin();
    }

    public void commit(){
        em.getTransaction().commit();
    }

    public void commitAndClear(){
        em.getTransaction().commit();
        em.clear();
    }

    public void commitAndClose(){
        em.getTransaction().commit();
        close();
    }

    public void rollbackAndClose(){
        rollback();
        close();
    }

    public void rollback(){
        if(em != null){
            EntityTransaction tx = em.getTransaction();
            if(tx != null && tx.isActive())
                tx.rollback();
        }
    }

    public void close(){
        if(em != null){
            em.close();
            em = null;
        }
    }

    public void closeAll(){
        if(em != null){
            EntityTransaction tx = em.getTransaction();
            if(tx != null && tx.isActive())
                tx.rollback();
            em.close();
        }
        if(emf != null)
            emf.close();
    }
    
    public static void println(String s){
        out.println(s);
    }
    
}
