
import javax.persistence.EntityManagerFactory;
import javax.persistence.Persistence;
import javax.persistence.EntityManager;
import javax.persistence.Query;

import java.sql.Connection;
import java.sql.SQLException;
import java.sql.Statement;
import java.util.List;
import java.util.Map;
import java.util.Vector;


/**
 * @author Wonseok Kim
 */
public class RefreshTest extends JpaTestCase {
    public static void main(String[] args) throws Exception {
        new RefreshTest().testCascadeRefresh();
    }

    public RefreshTest() {
    }

    public void testCascadeRefresh() throws SQLException {
        EntityManager em = createEntityManager();
        
        Person p1 = new Person(1, "Kim");
        Address address = new Address(1, "No address");
        p1.setAddress(address);
        
        em.getTransaction().begin();
        em.persist(p1);
        em.getTransaction().commit();
        em.clear();
        
        // modify externally
        em.getTransaction().begin();
        em.createNativeQuery("UPDATE DVT_PERSON SET NAME = ? WHERE ID=?")
            .setParameter(1, "Kim2").setParameter(2, 1).executeUpdate();
        em.createNativeQuery("UPDATE DVT_ADDRESS SET ADDRESS = ? WHERE ID=?")
            .setParameter(1, "New address").setParameter(2, 1).executeUpdate();
        em.getTransaction().commit();
        
        em.getTransaction().begin();
        Person e;
        e = em.find(Person.class, p1.getId());
        System.out.println("Name=" + e.getName() + ", Address=" + e.getAddress().address);
        
        em.refresh(e);
        
        System.out.println("Name=" + e.getName() + ", Address=" + e.getAddress().address);
        
        em.getTransaction().commit();
        
        assert "Kim2".equals(e.getName());
        assert "New address".equals(e.getAddress().address);
    }

    
}
